/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */

/* Copyright (c) Chris Vine, 2011 to 2023
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import Clutter from "gi://Clutter";
import St from "gi://St";
import GLib from "gi://GLib";
import Gio from "gi://Gio";

import * as Main from "resource:///org/gnome/shell/ui/main.js";
import * as PanelMenu from "resource:///org/gnome/shell/ui/panelMenu.js";
import * as PopupMenu from "resource:///org/gnome/shell/ui/popupMenu.js";
import * as AppFavorites from "resource:///org/gnome/shell/ui/appFavorites.js";

import {Extension} from "resource:///org/gnome/shell/extensions/extension.js";

export default class Favorites extends Extension {
    
    constructor(metadata) {
        super(metadata);

        this._settings_id = null;
        this._settings = null;
        this._button = null;
        this._menu_actor = null;


        // disable() can be called by the system before the timer in
        // enable() has fired.  We need to disconnect the timer in
        // disable() where this happens
        this._timer_id = null;
    }

    set_panel_display() {
        if (this._menu_actor) this._button.actor.remove_actor(this._menu_actor);
        
        if (this._settings.get_boolean("icon")) {
            let fpath = Gio.file_new_for_path(this.metadata["path"] +
                                              "/emblem-favorite.png");
            let icon = new St.Icon({gicon: new Gio.FileIcon({file: fpath}),
                                    style_class: "favorites-icon"});
            this._button.actor.add_actor(icon);
            this._menu_actor = icon;
        }
        else {
            let label = new St.Label({text: this.gettext("Favorites"),
				      y_expand: true,
				      y_align: Clutter.ActorAlign.CENTER
				     });
            this._button.actor.add_actor(label);
            this._menu_actor = label;
        }
    }

    add_item(app) {
        let item = new PopupMenu.PopupBaseMenuItem;
        this._button.menu.addMenuItem(item);
        let box = new St.BoxLayout({vertical: false,
				    pack_start: false,
				    style_class: "favorites-menu-box"});
        item.actor.add_child(box);
        let icon = app.create_icon_texture(24);
        box.add(icon);
        let label = new St.Label({text: app.get_name()});
        box.add(label);
        item.connect("activate", () => {app.open_new_window(-1);});
    }

    build_menu() {
        AppFavorites.getAppFavorites().getFavorites().forEach((app) => {
            this.add_item(app);
        });
    }

    enable() {
        // A one-shot timer of 100mS duration so that our panel button
        // is the last to be inserted in the panel's left box.  This
        // is necessary to ensure that the user's chosen position is
        // respected.  This does not give rise to any race condition
        // with respect to the user's actions on the GUI, because
        // until the timeout callback executes the favorites menu
        // cannot be acted on by the user (it is not in the panel and
        // the changed signal has not been connected).
        this._timer_id = GLib.timeout_add(GLib.PRIORITY_DEFAULT, 100, () => {
	    this._timer_id = null;
	    this._settings = this.getSettings();
	    this._button = new PanelMenu.Button(0.0);
	    this.set_panel_display();
	    this.build_menu();

	    this._settings_id =
                global.settings.connect("changed::" + AppFavorites.getAppFavorites().FAVORITE_APPS_KEY,
					() => {
					    this._button.menu.removeAll();
					    this.build_menu();
					});

	    Main.panel.addToStatusArea("favorites", this._button,
                                       this._settings.get_int("position") - 1, "left");

	    this._settings.connect("changed::icon", () => {
	        this.set_panel_display();
	    });
	    this._settings.connect("changed::position", () => {
	        let pos = this._settings.get_int("position") - 1;
	   
	        this._button.destroy();
	        this._button = new PanelMenu.Button(0.0);
	        this.set_panel_display();
	        this.build_menu();
	        Main.panel.addToStatusArea("favorites", this._button, pos, "left");
	    });
	    return false;
        });
    }

    disable() {
        if (this._settings_id) global.settings.disconnect(this._settings_id);
        if (this._timer_id) GLib.source_remove(this._timer_id);
        if (this._button) this._button.destroy();
        this._settings_id = null;
        this._timer_id = null;
        this._settings = null;
        this._button = null;
        this._menu_actor = null;
    }
}
